@@ -1,22 +1,26 @@
 import itertools
+import json
 import shutil
+from pathlib import Path
 from typing import Dict, Iterable, Optional, Sequence
 
-from pkg_resources import safe_name
-
 import click
 import halo
+import pythonfinder
 import tomlkit
+from pkg_resources import safe_name
+
 from pdm.builders import SdistBuilder, WheelBuilder
 from pdm.context import context
-from pdm.exceptions import ProjectError
+from pdm.exceptions import NoPythonVersion, ProjectError
 from pdm.installers import Synchronizer, format_dist
 from pdm.models.candidates import Candidate, identify
-from pdm.models.requirements import parse_requirement, strip_extras
+from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import bump_version, get_specifier
 from pdm.project import Project
 from pdm.resolver import BaseProvider, EagerUpdateProvider, ReusePinProvider, resolve
 from pdm.resolver.reporters import SpinnerReporter
+from pdm.utils import get_python_version
 
 
 def format_lockfile(mapping, fetched_dependencies, summary_collection):
@@ -56,6 +60,29 @@ def format_lockfile(mapping, fetched_dependencies, summary_collection):
     return doc
 
 
+def save_version_specifiers(
+    requirements: Dict[str, Requirement],
+    resolved: Dict[str, Candidate],
+    save_strategy: str,
+) -> None:
+    """Rewrite the version specifiers according to the resolved result and save strategy
+
+    :param requirements: the requirements to be updated
+    :param resolved: the resolved mapping
+    :param save_strategy: compatible/wildcard/exact
+    """
+    for name, r in requirements.items():
+        if r.is_named and not r.specifier:
+            if save_strategy == "exact":
+                r.specifier = get_specifier(f"=={resolved[name].version}")
+            elif save_strategy == "compatible":
+                version = str(resolved[name].version)
+                next_major_version = ".".join(
+                    map(str, bump_version(tuple(version.split(".")), 0))
+                )
+                r.specifier = get_specifier(f">={version},<{next_major_version}")
+
+
 def check_project_file(project: Project) -> None:
     """Check the existence of the project file and throws an error on failure."""
     if not project.tool_settings:
@@ -69,19 +96,22 @@ def do_lock(
     project: Project,
     strategy: str = "all",
     tracked_names: Optional[Iterable[str]] = None,
+    requirements: Optional[Dict[str, Dict[str, Requirement]]] = None,
 ) -> Dict[str, Candidate]:
     """Performs the locking process and update lockfile.
 
     :param project: the project instance
     :param strategy: update stratege: reuse/eager/all
     :param tracked_names: required when using eager strategy
+    :param requirements: An optional dictionary of requirements, read from pyproject
+        if not given.
     """
     check_project_file(project)
     # TODO: multiple dependency definitions for the same package.
     repository = project.get_repository()
-    requirements = project.all_dependencies
+    requirements = requirements or project.all_dependencies
     allow_prereleases = project.allow_prereleases
-    requires_python = project.python_requires
+    requires_python = project.environment.python_requires
     if strategy == "all":
         provider = BaseProvider(repository, requires_python, allow_prereleases)
     else:
@@ -183,21 +213,13 @@ def do_add(
         f"Adding packages to {section} dependencies: "
         + ", ".join(str(context.io.green(key, bold=True)) for key in requirements)
     )
-    project.add_dependencies(requirements)
-    resolved = do_lock(project, strategy, tracked_names)
-    for name in tracked_names:
-        r = requirements[name]
-        if r.is_named and not r.specifier:
-            if save == "exact":
-                r.specifier = get_specifier(f"=={resolved[name].version}")
-            elif save == "compatible":
-                version = str(resolved[name].version)
-                next_major_version = ".".join(
-                    map(str, bump_version(tuple(version.split(".")), 0))
-                )
-                r.specifier = get_specifier(f">={version},<{next_major_version}")
+    all_dependencies = project.all_dependencies
+    all_dependencies.setdefault(section, {}).update(requirements)
+    resolved = do_lock(project, strategy, tracked_names, all_dependencies)
+
     # Update dependency specifiers and lockfile hash.
-    project.add_dependencies(requirements, False)
+    save_version_specifiers(requirements, resolved, save)
+    project.add_dependencies(requirements)
     lockfile = project.lockfile
     lockfile["root"]["content_hash"] = "md5:" + project.get_content_hash("md5")
     project.write_lockfile(lockfile, False)
@@ -219,6 +241,8 @@ def do_update(
     sections: Sequence[str] = (),
     default: bool = True,
     strategy: str = "reuse",
+    save: str = "compatible",
+    unconstrained: bool = False,
     packages: Sequence[str] = (),
 ) -> None:
     """Update specified packages or all packages
@@ -228,6 +252,8 @@ def do_update(
     :param sections: update speicified sections
     :param default: update default
     :param strategy: update strategy (reuse/eager)
+    :param save: save strategy (compatible/exact/wildcard)
+    :param unconstrained: ignore version constraint
     :param packages: specified packages to update
     :return: None
     """
@@ -237,12 +263,18 @@ def do_update(
             "packages argument can't be used together with multple -s or --no-default."
         )
     if not packages:
+        if unconstrained:
+            raise click.BadArgumentUsage(
+                "--unconstrained must be used with package names given."
+            )
         # pdm update with no packages given, same as 'lock' + 'sync'
         do_lock(project)
         do_sync(project, sections, dev, default, clean=False)
         return
     section = sections[0] if sections else ("dev" if dev else "default")
-    dependencies = project.get_dependencies(section)
+    all_dependencies = project.all_dependencies
+    dependencies = all_dependencies[section]
+    updated_deps = {}
     tracked_names = set()
     for name in packages:
         matched_name = next(
@@ -259,14 +291,24 @@ def do_update(
                     context.io.green(name, bold=True), section
                 )
             )
+        if unconstrained:
+            dependencies[matched_name].specifier = get_specifier("")
         tracked_names.add(matched_name)
+        updated_deps[matched_name] = dependencies[matched_name]
     context.io.echo(
         "Updating packages: {}.".format(
             ", ".join(context.io.green(v, bold=True) for v in tracked_names)
         )
     )
-    do_lock(project, strategy, tracked_names)
+    resolved = do_lock(project, strategy, tracked_names, all_dependencies)
     do_sync(project, sections=(section,), default=False, clean=False)
+    if unconstrained:
+        # Need to update version constraints
+        save_version_specifiers(updated_deps, resolved, save)
+        project.add_dependencies(updated_deps)
+        lockfile = project.lockfile
+        lockfile["root"]["content_hash"] = "md5:" + project.get_content_hash("md5")
+        project.write_lockfile(lockfile, False)
 
 
 def do_remove(
@@ -302,9 +344,7 @@ def do_remove(
     for name in packages:
         matched_name = next(
             filter(
-                lambda k: safe_name(strip_extras(k)[0]).lower()
-                == safe_name(name).lower(),
-                deps.keys(),
+                lambda k: safe_name(k).lower() == safe_name(name).lower(), deps.keys(),
             ),
             None,
         )
@@ -322,14 +362,26 @@ def do_remove(
         do_sync(project, sections=(section,), default=False, clean=True)
 
 
-def do_list(project: Project) -> None:
+def do_list(project: Project, graph: bool = False) -> None:
+    """Display a list of packages installed in the local packages directory.
+
+    :param project: the project instance.
+    :param graph: whether to display a graph.
+    """
+    from pdm.cli.dependencies import build_dependency_graph, format_dependency_graph
+
     check_project_file(project)
     working_set = project.environment.get_working_set()
-    rows = [
-        (context.io.green(k, bold=True), format_dist(v))
-        for k, v in sorted(working_set.items())
-    ]
-    context.io.display_columns(rows, ["Package", "Version"])
+    if graph:
+        with project.environment.activate():
+            dep_graph = build_dependency_graph(working_set)
+        context.io.echo(format_dependency_graph(dep_graph))
+    else:
+        rows = [
+            (context.io.green(k, bold=True), format_dist(v))
+            for k, v in sorted(working_set.items())
+        ]
+        context.io.display_columns(rows, ["Package", "Version"])
 
 
 def do_build(
@@ -340,6 +392,8 @@ def do_build(
     clean: bool = True,
 ):
     """Build artifacts for distribution."""
+    if project.is_global:
+        raise ProjectError("Not allowed to build based on the global project.")
     check_project_file(project)
     if not wheel and not sdist:
         context.io.echo("All artifacts are disabled, nothing to do.", err=True)
@@ -363,6 +417,7 @@ def do_init(
     license: str = "MIT",
     author: str = "",
     email: str = "",
+    python_requires: str = "",
 ) -> None:
     """Bootstrap the project and create a pyproject.toml"""
     data = {
@@ -380,9 +435,88 @@ def do_init(
         },
         "build-system": {"requires": ["pdm"], "build-backend": "pdm.builders.api"},
     }
+    if python_requires and python_requires != "*":
+        data["tool"]["pdm"]["python_requires"] = python_requires
     if not project.pyproject:
         project._pyproject = data
     else:
         project._pyproject.setdefault("tool", {})["pdm"] = data["tool"]["pdm"]
         project._pyproject["build-system"] = data["build-system"]
     project.write_pyproject()
+
+
+def do_use(project: Project, python: str, first: bool = False) -> None:
+    """Use the specified python version and save in project config.
+    The python can be a version string or interpreter path.
+    """
+    if python and not all(c.isdigit() for c in python.split(".")):
+        if Path(python).exists():
+            python_path = Path(python).absolute().as_posix()
+        else:
+            python_path = shutil.which(python)
+        if not python_path:
+            raise NoPythonVersion(f"{python} is not a valid Python.")
+        python_version = get_python_version(python_path, True)
+    else:
+        finder = pythonfinder.Finder()
+        pythons = []
+        args = [int(v) for v in python.split(".") if v != ""]
+        for i, entry in enumerate(finder.find_all_python_versions(*args)):
+            python_version = get_python_version(entry.path.as_posix(), True)
+            pythons.append((entry.path.as_posix(), python_version))
+        if not pythons:
+            raise NoPythonVersion(f"Python {python} is not available on the system.")
+
+        if not first and len(pythons) > 1:
+            for i, (path, python_version) in enumerate(pythons):
+                context.io.echo(f"{i}: {context.io.green(path)} ({python_version})")
+            selection = click.prompt(
+                "Please select:",
+                type=click.Choice([str(i) for i in range(len(pythons))]),
+                default="0",
+                show_choices=False,
+            )
+        else:
+            selection = 0
+        python_path, python_version = pythons[int(selection)]
+
+    if not project.python_requires.contains(python_version):
+        raise NoPythonVersion(
+            "The target Python version {} doesn't satisfy "
+            "the Python requirement: {}".format(python_version, project.python_requires)
+        )
+    context.io.echo(
+        "Using Python interpreter: {} ({})".format(
+            context.io.green(python_path), python_version
+        )
+    )
+
+    project.project_config["python.path"] = Path(python_path).as_posix()
+
+
+def do_info(
+    project: Project,
+    python: bool = False,
+    show_project: bool = False,
+    env: bool = False,
+) -> None:
+    """Show project information."""
+    python_path = project.environment.python_executable
+    python_version = get_python_version(python_path, True)
+    if not python and not show_project and not env:
+        rows = [
+            (
+                context.io.cyan("Python Interpreter:", bold=True),
+                python_path + f" ({python_version})",
+            ),
+            (context.io.cyan("Project Root:", bold=True), project.root.as_posix()),
+        ]
+        context.io.display_columns(rows)
+        return
+
+    if python:
+        context.io.echo(python_path)
+    if show_project:
+        context.io.echo(project.root.as_posix())
+    if env:
+        context.io.echo(json.dumps(project.environment.marker_environment, indent=2))
