@@ -11,27 +11,29 @@ from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Tuple
 
 from pip._internal.req import req_uninstall
 from pip._internal.utils import misc
-from pip._vendor import pkg_resources
+from pip._vendor import packaging, pkg_resources
 from pip_shims import shims
+from pythonfinder import Finder
+from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
+from vistir.contextmanagers import temp_environ
+from vistir.path import normalize_path
 
 from pdm.context import context
 from pdm.exceptions import NoPythonVersion
+from pdm.models.specifiers import PySpecSet
 from pdm.utils import (
-    _allow_all_wheels,
+    allow_all_wheels,
     cached_property,
     convert_hashes,
     create_tracked_tempdir,
     get_finder,
     get_pep508_environment,
     get_python_version,
+    get_sys_config_paths,
 )
-from pythonfinder import Finder
-from vistir.contextmanagers import temp_environ
-from vistir.path import normalize_path
 
 if TYPE_CHECKING:
-    from pdm.models.specifiers import PySpecSet
-    from pdm.project.config import Config
+    from pdm.project import Project
     from pdm._types import Source
 
 
@@ -71,33 +73,66 @@ class WorkingSet(collections.abc.Mapping):
 
 
 class Environment:
-    def __init__(self, python_requires: PySpecSet, config: Config) -> None:
-        self.python_requires = python_requires
-        self.config = config
+    """Environment dependent stuff related to the selected Python interpreter."""
+
+    is_global = False
+
+    def __init__(self, project: Project) -> None:
+        """
+        :param project: the project instance
+        """
+        self.python_requires = project.python_requires
+        self.config = project.config
+        self.project = project
 
     @cached_property
     def python_executable(self) -> str:
         """Get the Python interpreter path."""
-        if self.config["python"]:
-            path = self.config["python"]
-            try:
-                get_python_version(path)
-                return path
-            except Exception:
-                pass
-        else:
+        if self.config.get("python.path"):
+            return self.config["python.path"]
+        if PYENV_INSTALLED and self.config.get("python.use_pyenv", True):
+            return os.path.join(PYENV_ROOT, "shims", "python")
+        if "VIRTUAL_ENV" in os.environ:
+            context.io.echo(
+                "An activated virtualenv is detected, reuse the interpreter now.",
+                err=True,
+                verbosity=context.io.DETAIL,
+            )
+            scripts_dir = "Scripts" if os.name == "nt" else "bin"
+            return os.path.join(os.environ["VIRTUAL_ENV"], scripts_dir, "python")
+
+        # First try what `python` refers to.
+        path = shutil.which("python")
+        version = None
+        if path:
+            version = get_python_version(path, True)
+        if not version or not self.python_requires.contains(version):
             finder = Finder()
             for python in finder.find_all_python_versions():
-                version = ".".join(map(str, get_python_version(python.path.as_posix())))
+                version = get_python_version(python.path.as_posix(), True)
                 if self.python_requires.contains(version):
-                    return python.path.as_posix()
-            if self.python_requires.contains(".".join(map(str, sys.version_info[:3]))):
-                return sys.executable
+                    path = python.path.as_posix()
+                    break
+            else:
+                version = ".".join(map(str, sys.version_info[:3]))
+                if self.python_requires.contains(version):
+                    path = sys.executable
+        if path:
+            context.io.echo(
+                "Using Python interpreter: {} ({})".format(
+                    context.io.green(path), version
+                )
+            )
+            self.project.project_config["python.path"] = Path(path).as_posix()
+            return path
         raise NoPythonVersion(
-            "No python matching {} is found on the system.".format(self.python_requires)
+            "No Python that satisfies {} is found on the system.".format(
+                self.python_requires
+            )
         )
 
     def get_paths(self) -> Dict[str, str]:
+        """Get paths like ``sysconfig.get_paths()`` for installation."""
         paths = sysconfig.get_paths()
         scripts = "Scripts" if os.name == "nt" else "bin"
         packages_path = self.packages_path
@@ -111,6 +146,9 @@ class Environment:
 
     @contextmanager
     def activate(self):
+        """Activate the environment. Manipulate the ``PYTHONPATH`` and patches ``pip``
+        to be aware of local packages. This method acts like a context manager.
+        """
         paths = self.get_paths()
         with temp_environ():
             old_paths = os.getenv("PYTHONPATH")
@@ -129,25 +167,37 @@ class Environment:
             # HACK: Replace the is_local with environment version so that packages can
             # be removed correctly.
             _old_sitepackages = misc.site_packages
+            misc.site_packages = paths["purelib"]
             _is_local = misc.is_local
             misc.is_local = req_uninstall.is_local = self.is_local
-            misc.site_packages = paths["purelib"]
+            _evaluate_marker = pkg_resources.evaluate_marker
+            pkg_resources.evaluate_marker = self.evaluate_marker
+            sys_executable = sys.executable
+            sys.executable = self.python_executable
             yield
+            sys.executable = sys_executable
+            pkg_resources.evaluate_marker = _evaluate_marker
             misc.is_local = req_uninstall.is_local = _is_local
-            pkg_resources.working_set = _old_ws
             misc.site_packages = _old_sitepackages
+            pkg_resources.working_set = _old_ws
 
     def is_local(self, path) -> bool:
+        """PEP 582 version of ``is_local()`` function."""
         return normalize_path(path).startswith(
             normalize_path(self.packages_path.as_posix())
         )
 
+    def evaluate_marker(self, text: str, extra=None) -> bool:
+        marker = packaging.markers.Marker(text)
+        return marker.evaluate(self.marker_environment)
+
     @cached_property
     def packages_path(self) -> Path:
+        """The local packages path."""
         if self.config.get("packages_path") is not None:
             return self.config.get("packages_path")
         pypackages = (
-            self.config.project_root
+            self.project.root
             / "__pypackages__"
             / ".".join(map(str, get_python_version(self.python_executable)[:2]))
         )
@@ -190,6 +240,11 @@ class Environment:
         sources: Optional[List[Source]] = None,
         ignore_requires_python: bool = False,
     ) -> shims.PackageFinder:
+        """Return the package finder of given index sources.
+
+        :param sources: a list of sources the finder should search in.
+        :param ignore_requires_python: whether to ignore the python version constraint.
+        """
         sources = sources or []
         python_version = get_python_version(self.python_executable)[:2]
         finder = get_finder(
@@ -202,16 +257,30 @@ class Environment:
         finder.session.close()
 
     def build(
-        self, ireq: shims.InstallRequirement, hashes: Optional[Dict[str, str]] = None
+        self,
+        ireq: shims.InstallRequirement,
+        hashes: Optional[Dict[str, str]] = None,
+        allow_all: bool = True,
     ) -> str:
-        """A local candidate has already everything in local, no need to download."""
+        """Build egg_info directory for editable candidates and a wheel for others.
+
+        :param ireq: the InstallRequirment of the candidate.
+        :param hashes: a dictionary of filename: hash_value to check against downloaded
+        artifacts.
+        :param allow_all: Allow building incompatible wheels.
+        :returns: The full path of the built artifact.
+        """
+        from pip._internal.utils.temp_dir import global_tempdir_manager
         from pdm.builders import EditableBuilder
         from pdm.builders import WheelBuilder
 
         kwargs = self._make_building_args(ireq)
         with self.get_finder() as finder:
-            with _allow_all_wheels():
-                # temporarily allow all wheels to get a link.
+            if allow_all:
+                with allow_all_wheels():
+                    # temporarily allow all wheels to get a link.
+                    ireq.populate_link(finder, False, bool(hashes))
+            else:
                 ireq.populate_link(finder, False, bool(hashes))
             if not ireq.editable and not ireq.req.name:
                 ireq.source_dir = kwargs["build_dir"]
@@ -219,24 +288,34 @@ class Environment:
                 ireq.ensure_has_source_dir(kwargs["build_dir"])
 
             download_dir = kwargs["download_dir"]
+            only_download = False
             if ireq.link.is_wheel:
                 download_dir = kwargs["wheel_download_dir"]
+                only_download = True
             if hashes:
                 ireq.options["hashes"] = convert_hashes(hashes)
             if not (ireq.editable and ireq.req.is_local_dir):
-                shims.shim_unpack(
-                    link=ireq.link,
-                    download_dir=download_dir,
-                    location=ireq.source_dir,
-                    hashes=ireq.hashes(False),
-                    session=finder.session,
-                )
+                with global_tempdir_manager():
+                    downloaded = shims.shim_unpack(
+                        link=ireq.link,
+                        download_dir=download_dir,
+                        location=ireq.source_dir,
+                        hashes=ireq.hashes(False),
+                        only_download=only_download,
+                        session=finder.session,
+                    )
+                    # Preserve the downloaded file so that it won't be cleared.
+                    if downloaded and only_download:
+                        try:
+                            shutil.copy(downloaded, download_dir)
+                        except shutil.SameFileError:
+                            pass
             # Now all source is prepared, build it.
             if ireq.link.is_wheel:
                 return (context.cache("wheels") / ireq.link.filename).as_posix()
             builder_class = EditableBuilder if ireq.editable else WheelBuilder
             kwargs["finder"] = finder
-            with builder_class(ireq) as builder:
+            with builder_class(ireq) as builder, self.activate():
                 return builder.build(**kwargs)
 
     def get_working_set(self) -> WorkingSet:
@@ -246,6 +325,7 @@ class Environment:
             [paths["platlib"]], python=get_python_version(self.python_executable)
         )
 
+    @cached_property
     def marker_environment(self) -> Dict[str, Any]:
         """Get environment for marker evaluation"""
         return get_pep508_environment(self.python_executable)
@@ -255,8 +335,35 @@ class Environment:
         if not os.path.isabs(command) and command.startswith("python"):
             python = os.path.splitext(command)[0]
             version = python[6:]
-            this_version = get_python_version(self.python_executable)
-            if not version or ".".join(map(str, this_version)).startswith(version):
+            this_version = get_python_version(self.python_executable, True)
+            if not version or this_version.startswith(version):
                 return self.python_executable
         # Fallback to use shutil.which to find the executable
         return shutil.which(command, path=os.getenv("PATH"))
+
+
+class GlobalEnvironment(Environment):
+    """Global environment"""
+
+    is_global = True
+
+    def __init__(self, project: Project) -> None:
+        super().__init__(project)
+        self.python_requires = PySpecSet(
+            "==" + get_python_version(self.python_executable, True)
+        )
+
+    def get_paths(self) -> Dict[str, str]:
+        paths = get_sys_config_paths(self.python_executable)
+        paths["prefix"] = paths["data"]
+        paths["headers"] = paths["include"]
+        return paths
+
+    def is_local(self, path) -> bool:
+        return normalize_path(path).startswith(
+            normalize_path(self.get_paths()["prefix"])
+        )
+
+    @property
+    def packages_path(self) -> Optional[Path]:
+        return None
