@@ -14,17 +14,15 @@ import tempfile
 import urllib.parse as parse
 from contextlib import contextmanager
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
-
-from pip_shims import Wheel as PipWheel
-from pip_shims.backports import get_session, resolve_possible_shim
-from pip_shims.shims import InstallCommand, PackageFinder, TargetPython
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple
 
 from distlib.wheel import Wheel
+from pip_shims.shims import InstallCommand, PackageFinder, TargetPython
+
 from pdm._types import Source
 
 if TYPE_CHECKING:
-    from pip_shims.backports import TCommand, TShimmedFunc, Values, TSession, TFinder
+    from pip_shims.compat import TCommand, TShimmedFunc, Values, TSession, TFinder
 
 try:
     from functools import cached_property
@@ -50,7 +48,7 @@ def get_abi_tag(python_version):
     (CPython 2, PyPy).
     A replacement for pip._internal.models.pep425tags:get_abi_tag()
     """
-    from pip._internal.pep425tags import get_config_var, get_abbr_impl, get_flag
+    from wheel.pep425tags import get_config_var, get_abbr_impl, get_flag
 
     soabi = get_config_var("SOABI")
     impl = get_abbr_impl()
@@ -61,19 +59,17 @@ def get_abi_tag(python_version):
         m = ""
         u = ""
         is_cpython = impl == "cp"
-        if get_flag(
-            "Py_DEBUG", lambda: hasattr(sys, "gettotalrefcount"), warn=is_cpython
-        ):
+        if get_flag("Py_DEBUG", lambda: hasattr(sys, "gettotalrefcount"), warn=False):
             d = "d"
         if python_version < (3, 8) and get_flag(
-            "WITH_PYMALLOC", lambda: is_cpython, warn=is_cpython
+            "WITH_PYMALLOC", lambda: is_cpython, warn=False
         ):
             m = "m"
         if python_version < (3, 3) and get_flag(
             "Py_UNICODE_SIZE",
             lambda: sys.maxunicode == 0x10FFFF,
             expected=4,
-            warn=is_cpython,
+            warn=False,
         ):
             u = "u"
         abi = "%s%s%s%s%s" % (impl, "".join(map(str, python_version)), d, m, u)
@@ -128,6 +124,8 @@ def get_package_finder(
     :return: A :class:`pip._internal.index.package_finder.PackageFinder` instance
     :rtype: :class:`pip._internal.index.package_finder.PackageFinder`
     """
+    from pip_shims.compat import get_session, resolve_possible_shim
+
     if install_cmd is None:
         install_cmd_provider = resolve_possible_shim(install_cmd_provider)
         assert isinstance(install_cmd_provider, (type, functools.partial))
@@ -218,7 +216,7 @@ def get_pypi_source():
     verify_ssl = parsed.scheme == "https"
     if any(parsed.hostname.startswith(host) for host in options.trusted_hosts):
         verify_ssl = False
-    return {"url": index_url, "name": "pypi", "verify_ssl": verify_ssl}
+    return {"pypi.url": index_url, "pypi.verify_ssl": verify_ssl}
 
 
 def get_finder(
@@ -289,7 +287,7 @@ def _wheel_support_index_min(self, tags=None):
 
 
 @contextmanager
-def _allow_all_wheels():
+def allow_all_wheels():
     """Monkey patch pip.Wheel to allow all wheels
 
     The usual checks against platforms and Python versions are ignored to allow
@@ -297,6 +295,8 @@ def _allow_all_wheels():
     and set a new one, or else the results from the previous non-patched calls
     will interfere.
     """
+    from pip._internal.models.wheel import Wheel as PipWheel
+
     original_wheel_supported = PipWheel.supported
     original_support_index_min = PipWheel.support_index_min
 
@@ -325,13 +325,28 @@ def find_project_root(cwd: str = ".", max_depth: int = 5) -> Optional[str]:
 
 
 @functools.lru_cache()
-def get_python_version(executable: str) -> Tuple[Union[str, int], ...]:
+def get_python_version(executable, as_string=False):
+    """Get the version of the Python interperter."""
     args = [
         executable,
         "-c",
         "import sys,json;print(json.dumps(tuple(sys.version_info[:3])))",
     ]
-    return tuple(json.loads(subprocess.check_output(args)))
+    result = tuple(json.loads(subprocess.check_output(args)))
+    if not as_string:
+        return result
+    return ".".join(map(str, result))
+
+
+@functools.lru_cache()
+def get_sys_config_paths(executable: str) -> Dict[str, str]:
+    """Return the sys_config.get_paths() result for the python interpreter"""
+    args = [
+        executable,
+        "-c",
+        "import sysconfig,json;print(json.dumps(sysconfig.get_paths()))",
+    ]
+    return json.loads(subprocess.check_output(args))
 
 
 def get_pep508_environment(executable: str) -> Dict[str, Any]:
@@ -376,3 +391,16 @@ def get_user_email_from_git() -> Tuple[str, str]:
     except subprocess.CalledProcessError:
         email = ""
     return username, email
+
+
+def add_ssh_scheme_to_git_uri(uri: str) -> str:
+    """Cleans VCS uris from pip format"""
+    # Add scheme for parsing purposes, this is also what pip does
+    if "://" not in uri:
+        uri = "ssh://" + uri
+        parsed = parse.urlparse(uri)
+        if ":" in parsed.netloc:
+            netloc, _, path_start = parsed.netloc.rpartition(":")
+            path = "/{0}{1}".format(path_start, parsed.path)
+            uri = parse.urlunparse(parsed._replace(netloc=netloc, path=path))
+    return uri
